@@ -41,12 +41,15 @@ knowledge of the CeCILL v2.1 license and that you accept its terms.
 #include <QMouseEvent>
 #include <QAbstractItemModel>
 #include <QAbstractItemView>
+#include <QTableView>
+#include <QTreeView>
 #include <QBuffer>
 #include <QDesktopWidget>
 #include <QGraphicsView>
 #include <QGraphicsItem>
 #include <QTimer>
 #include <QTime>
+#include <QHeaderView>
 #include "dragndropresponse.h"
 #include "shortcutresponse.h"
 
@@ -81,6 +84,11 @@ void mouse_dclick(QWidget * w, const QPoint & pos) {
                         Qt::NoModifier));
 }
 
+void activate_focus(QWidget * w) {
+    w->activateWindow();
+    w->setFocus(Qt::MouseFocusReason);
+}
+
 void dump_properties(QObject * object, QtJson::JsonObject & out) {
     const QMetaObject * metaobject = object->metaObject();
     for (int i = 0; i < metaobject->propertyCount(); ++i) {
@@ -168,7 +176,7 @@ void dump_items_model(QAbstractItemModel * model,
             QModelIndex index = model->index(i, j, parent);
             QtJson::JsonObject item;
             dump_item_model_attrs(model, item, index, viewid);
-            if (recursive && model->hasChildren(index)) {
+            if (j == 0 && recursive && model->hasChildren(index)) {
                 dump_items_model(model, item, index, viewid);
             }
             items << item;
@@ -217,7 +225,11 @@ void dump_graphics_items(const QList<QGraphicsItem *>  & items, const qulonglong
             outitem["classes"] = classes;
             outitem["objectname"] = itemObject->objectName();
         }
+        #if (QT_VERSION >= QT_VERSION_CHECK(5,0,0))
+        dump_graphics_items(item->childItems(), viewid, outitem);
+        #else
         dump_graphics_items(item->children(), viewid, outitem);
+        #endif
         outitems << outitem;
     }
     out["items"] = outitems;
@@ -261,7 +273,11 @@ QtJson::JsonObject Player::list_actions(const QtJson::JsonObject &) {
     for(int i = metaObject->methodOffset(); i < metaObject->methodCount(); ++i) {
         QMetaMethod method = metaObject->method(i);
         if (method.methodType() == QMetaMethod::Slot) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5,0,0))
+            methods << QString::fromLatin1(metaObject->method(i).methodSignature());
+#else
             methods << QString::fromLatin1(metaObject->method(i).signature());
+#endif
         }
     }
     QtJson::JsonObject result;
@@ -274,7 +290,7 @@ QtJson::JsonObject Player::widget_by_path(const QtJson::JsonObject & command) {
     QObject * o = findObject(path);
     qulonglong id = registerObject(o);
     if (id == 0) {
-        return createError("InvalidWidgetPath", QString("Le widget est introuvable pour le path `%1`").arg(path));
+        return createError("InvalidWidgetPath", QString("Unable to find widget with path `%1`").arg(path));
     }
     QtJson::JsonObject result;
     result["oid"] = id;
@@ -297,7 +313,7 @@ QtJson::JsonObject Player::active_widget(const QtJson::JsonObject & command) {
     if (! active) {
         type = "window";
         return createError("NoActiveWindow",
-                            QString::fromUtf8("Il n'y a pas de widget (%1) actif").arg(type));
+                            QString::fromUtf8("There is no active widget (%1)").arg(type));
     }
     qulonglong id = registerObject(active);
     QtJson::JsonObject result;
@@ -313,7 +329,7 @@ ObjectLocatorContext::ObjectLocatorContext(Player * player,
     obj = player->registeredObject(id);
     if (!obj) {
         lastError = player->createError("NotRegisteredObject",
-                           QString::fromUtf8("L'objet (id:%1) n'est pas enregistré ou a été détruit").arg(id));
+                           QString::fromUtf8("The object (id:%1) is not registered or has been destroyed").arg(id));
     }
 }
 
@@ -411,7 +427,7 @@ QtJson::JsonObject Player::model_items(const QtJson::JsonObject & command) {
     QtJson::JsonObject result;
     QAbstractItemModel * model = ctx.widget->model();
     if (!model) {
-        return createError("MissingModel", QString::fromUtf8("La vue (id:%1) ne possède pas de modèle.").arg(ctx.id));
+        return createError("MissingModel", QString::fromUtf8("The view (id:%1) has no associated model").arg(ctx.id));
     }
     bool recursive = ! (model->inherits("QTableModel") || ctx.widget->inherits("QTableView") || ctx.widget->inherits("QListView"));
     dump_items_model(model, result, QModelIndex(), ctx.id, recursive);
@@ -423,26 +439,56 @@ QtJson::JsonObject Player::model_item_action(const QtJson::JsonObject & command)
     if (ctx.hasError()) { return ctx.lastError; }
     QAbstractItemModel * model = ctx.widget->model();
     if (!model) {
-        return createError("MissingModel", QString::fromUtf8("La vue (id:%1) ne possède pas de modèle.").arg(ctx.id));
+        return createError("MissingModel", QString::fromUtf8("The view (id:%1) has no associated model").arg(ctx.id));
     }
     QModelIndex index = get_model_item(model, command["itempath"].toString(), command["row"].toInt(), command["column"].toInt());
     if (!index.isValid()) {
-        return createError("MissingModelItem", QString::fromUtf8("Impossible de trouver l'item identifié par %1").arg(command["itempath"].toString()));
+        return createError("MissingModelItem", QString::fromUtf8("Unable to find an item identified by %1").arg(command["itempath"].toString()));
     }
     ctx.widget->scrollTo(index); // item visible
     QString itemaction = command["itemaction"].toString();
+    
+    QPoint cursorPosition;
+
+    if (itemaction == "click" || itemaction == "doubleclick") {
+        QString origin = command["origin"].toString();
+        int offsetX = command["offset_x"].toInt();
+        int offsetY = command["offset_y"].toInt();
+        QRect visualRect = ctx.widget->visualRect(index);
+        cursorPosition = visualRect.center();
+        if (origin == "left") {
+            cursorPosition.setX(visualRect.x());
+        } else if (origin == "right") {
+            cursorPosition.setX(visualRect.width() - 1);
+        }
+        int newX = cursorPosition.x() + offsetX;
+        int newY = cursorPosition.y() + offsetY;
+
+        /* The new coordinates have to be within the bounds */
+        if (newX < visualRect.x())
+            newX = visualRect.x() + 2;
+        else if (newX > visualRect.x() + visualRect.width())
+            newX = visualRect.x() + visualRect.width() - 2;
+
+        if (newY < visualRect.y())
+            newY = visualRect.y() + 2;
+        else if (newY > visualRect.y() + visualRect.height())
+            newY = visualRect.y() + visualRect.height() - 2;
+
+        cursorPosition.setX(newX);
+        cursorPosition.setY(newY);
+    }
+
     if (itemaction == "select") {
         emit emit_model_item_action(itemaction, ctx.widget, index);
     } else if (itemaction == "edit") {
         emit emit_model_item_action(itemaction, ctx.widget, index);
     } else if (itemaction == "click") {
-        QRect visualRect = ctx.widget->visualRect(index);
-        mouse_click(ctx.widget->viewport(), visualRect.center());
+        mouse_click(ctx.widget->viewport(), cursorPosition);
     } else if (itemaction == "doubleclick") {
-        QRect visualRect = ctx.widget->visualRect(index);
-        mouse_dclick(ctx.widget->viewport(), visualRect.center());
+        mouse_dclick(ctx.widget->viewport(), cursorPosition);
     } else {
-        return createError("MissingItemAction", QString::fromUtf8("itemaction %1 inconnue").arg(itemaction));
+        return createError("MissingItemAction", QString::fromUtf8("itemaction %1 unknown").arg(itemaction));
     }
     QtJson::JsonObject result;
     return result;
@@ -462,9 +508,9 @@ QtJson::JsonObject Player::model_gitem_action(const QtJson::JsonObject & command
     if (ctx.hasError()) { return ctx.lastError; }
     QGraphicsItem * item = graphicsItemFromPath(ctx.widget, command["stackpath"].toString());
     if (!item) {
-        return createError("MissingModel", QString::fromUtf8("La vue (id:%1) ne possède pas de modèle.").arg(ctx.id));
+        return createError("MissingModel", QString::fromUtf8("The view (id:%1) has no associated model").arg(ctx.id));
     }
-    ctx.widget->ensureVisible(item); // pour rendre l'item visible
+    ctx.widget->ensureVisible(item); // be sure item is visible
     QString itemaction = command["itemaction"].toString();
 
     QPoint viewPos = ctx.widget->mapFromScene(item->mapToScene(item->boundingRect().center()));
@@ -479,7 +525,7 @@ QtJson::JsonObject Player::model_gitem_action(const QtJson::JsonObject & command
         }
         mouse_dclick(ctx.widget->viewport(), viewPos);
     } else {
-        return createError("MissingItemAction", QString::fromUtf8("itemaction %1 inconnue").arg(itemaction));
+        return createError("MissingItemAction", QString::fromUtf8("itemaction %1 unknown").arg(itemaction));
     }
     QtJson::JsonObject result;
     return result;
@@ -512,7 +558,7 @@ QtJson::JsonObject Player::widget_keyclick(const QtJson::JsonObject & command) {
     QString text = command["text"].toString();
     for (int i=0; i<text.count(); ++i) {
         QChar ch = text[i];
-        int key = (int) ch.toAscii();
+        int key = (int) ch.toLatin1();
         qApp->postEvent(widget, new QKeyEvent(QKeyEvent::KeyPress, key, Qt::NoModifier, ch));
         qApp->postEvent(widget, new QKeyEvent(QKeyEvent::KeyRelease, key, Qt::NoModifier, ch));
     }
@@ -536,6 +582,94 @@ QtJson::JsonObject Player::tabbar_list(const QtJson::JsonObject & command) {
     return result;
 }
 
+QtJson::JsonObject Player::headerview_list(const QtJson::JsonObject & command) {
+    WidgetLocatorContext<QHeaderView> ctx(this, command, "oid");
+    if (ctx.hasError()) { return ctx.lastError; }
+    QAbstractItemModel * model = ctx.widget->model();
+    if (!model) {
+        return createError("MissingModel", QString::fromUtf8("The header view (id:%1) has no associated model").arg(ctx.id));
+    }
+    QStringList texts;
+    int nbItems = ctx.widget->orientation() == Qt::Vertical ? model->rowCount() : model->columnCount();
+    for (int i=0; i<nbItems; i++) {
+        texts << model->headerData(i, ctx.widget->orientation()).toString();
+    }
+    QtJson::JsonObject result;
+    result["headertexts"] = texts;
+    return result;
+}
+
+QtJson::JsonObject Player::headerview_click(const QtJson::JsonObject & command) {
+    WidgetLocatorContext<QHeaderView> ctx(this, command, "oid");
+    if (ctx.hasError()) { return ctx.lastError; }
+    int logicalIndex;
+    QVariant indexOrName = command["indexOrName"];
+    if (indexOrName.type() == QVariant::String) {
+        QString name = indexOrName.toString();
+        QAbstractItemModel * model = ctx.widget->model();
+        if (!model) {
+            return createError("MissingModel", QString::fromUtf8("The header view (id:%1) has no associated model").arg(ctx.id));
+        }
+        bool found = false;
+        int nbItems = ctx.widget->orientation() == Qt::Horizontal ? model->rowCount() : model->columnCount();
+        for (int i=0; i<nbItems; i++) {
+            if (name == model->headerData(i, ctx.widget->orientation()).toString()) {
+                logicalIndex = i;
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            return createError("MissingHeaderViewText", QString::fromUtf8("The header view (id:%1) has no text column `%2`").arg(ctx.id).arg(name));
+        }
+    } else {
+        logicalIndex = ctx.widget->logicalIndex(command["indexOrName"].toInt());
+    }
+    
+    int pos = ctx.widget->sectionPosition(logicalIndex);
+    if (pos == -1) {
+        return createError("InvalidHeaderViewIndex", QString::fromUtf8("The header view (id:%1) has no index %2 or it is hidden").arg(ctx.id).arg(logicalIndex));
+    }
+    QPoint mousePos;
+    if (ctx.widget->orientation() == Qt::Horizontal) {
+        mousePos.setY(ctx.widget->height()/2);
+        mousePos.setX(pos + ctx.widget->offset() + 5);
+    } else {
+        mousePos.setX(ctx.widget->width()/2);
+        mousePos.setY(pos + ctx.widget->offset() + 5);
+    }
+    mouse_click(ctx.widget->viewport(), mousePos);
+    QtJson::JsonObject result;
+    return result;
+}
+
+QtJson::JsonObject Player::headerview_path_from_view(const QtJson::JsonObject & command) {
+    WidgetLocatorContext<QAbstractItemView> ctx(this, command, "oid");
+    if (ctx.hasError()) { return ctx.lastError; }
+    
+    QHeaderView * header = NULL;
+    QTableView * tview = qobject_cast<QTableView *>(ctx.widget);
+    if (tview) {
+        if (command["orientation"] == "vertical") {
+            header = tview->verticalHeader();
+        } else {
+            header = tview->horizontalHeader();
+        }
+    } else {
+        QTreeView * trview = qobject_cast<QTreeView *>(ctx.widget);
+        if (trview) {
+            header = trview->header();
+        }
+    }
+    
+    if (! header) {
+        return createError("InvalidHeaderView", QString::fromUtf8("No header view found for the view (id:%1)").arg(ctx.id));
+    }
+    QtJson::JsonObject result;
+    result["headerpath"] = objectPath(header);
+    return result;
+}
+
 QtJson::JsonObject Player::graphicsitems(const QtJson::JsonObject & command) {
     WidgetLocatorContext<QGraphicsView> ctx(this, command, "oid");
     if (ctx.hasError()) { return ctx.lastError; }
@@ -556,12 +690,12 @@ QtJson::JsonObject Player::gitem_properties(const QtJson::JsonObject & command)
     QString stackpath = command["stackpath"].toString();
     QGraphicsItem * item = graphicsItemFromPath(ctx.widget, stackpath);
     if (!item) {
-        return createError("MissingGItem", QString::fromUtf8("QGraphicsitem %1 introuvable sur la vue %2")
+        return createError("MissingGItem", QString::fromUtf8("QGraphicsitem %1 is not in view %2")
                            .arg(stackpath).arg(ctx.id));
     }
     QObject * object = dynamic_cast<QObject *>(item);
     if (!object) {
-        return createError("GItemNotQObject", QString::fromUtf8("QGraphicsitem %1 sur la vue %2 n'hérite pas de QObject")
+        return createError("GItemNotQObject", QString::fromUtf8("QGraphicsitem %1 in view %2 does not inherit from QObject")
                            .arg(stackpath).arg(ctx.id));
     }
     QtJson::JsonObject result;
@@ -583,7 +717,7 @@ QtJson::JsonObject Player::call_slot(const QtJson::JsonObject & command) {
                                       Q_RETURN_ARG(QVariant, result_slot),
                                       Q_ARG(QVariant, command["params"]));
     if (!invokedMeth) {
-        return createError("NoMethodInvoked", QString::fromUtf8("Le slot %1 n'a pas pu être appelé")
+        return createError("NoMethodInvoked", QString::fromUtf8("The slot %1 could not be called")
                            .arg(slot_name));
     }
 
@@ -591,3 +725,12 @@ QtJson::JsonObject Player::call_slot(const QtJson::JsonObject & command) {
     result["result_slot"] = result_slot;
     return result;
 }
+
+QtJson::JsonObject Player::widget_activate_focus(const QtJson::JsonObject & command) {
+    WidgetLocatorContext<QWidget> ctx(this, command, "oid");
+    if (ctx.hasError()) { return ctx.lastError; }
+    activate_focus(ctx.widget);
+
+    QtJson::JsonObject result;
+    return result;
+}
